<?php


namespace Puddinq;

use Composer\Composer;
use Composer\IO\IOInterface;
use Composer\Script\Event;

/**
 * Class Setup
 *
 * https://getcomposer.org/doc/articles/scripts.md#defining-scripts
 *
 * @package Puddinq
 */

class Setup
{

    private static $CONFIG_KEYS = array(
        'WP_ENV',
        'DB_NAME',
        'DB_USER',
        'DB_PASSWORD',
        'DB_HOST',
        'DB_PREFIX',
        'WP_HOME',
        'WP_MEMORY_LIMIT',
    );

    private static $AUTH_KEYS = array(
        'AUTH_KEY',
        'SECURE_AUTH_KEY',
        'LOGGED_IN_KEY',
        'NONCE_KEY',
        'AUTH_SALT',
        'SECURE_AUTH_SALT',
        'LOGGED_IN_SALT',
        'NONCE_SALT',
    );

    /**
     * @var string
     */
    private static $root;

    /**
     * @var Composer
     */
    private static $composer;

    /**
     * @var IOInterface
     */
    private static $io;

    /**
     * Performed after the 'composer update' command.
     *
     * @param Event $event
     */
    public static function activate(Event $event)
    {
        Setup::defaultActions($event);

        //Remove WPs default content directory with standard plugins and themes
        Setup::removeDefaultContentDir();

        if (Setup::$io->isInteractive()) {
            //Create config file if IO is interactive
            Setup::createConfigFile();
        }
    }

    /**
     * The default actions to take after each composer command.
     *
     * @param Event $event
     */
    private static function defaultActions(Event $event)
    {
        Setup::$root = dirname(dirname(__DIR__));
        self::$composer = $event->getComposer();
        Setup::$io = $event->getIO();
        Setup::copyHtAccessFile();

        //Move default language files to our own content dir
        Setup::moveLanguageFolder();
    }

    /**
     * Move the language folder.
     */
    private static function moveLanguageFolder()
    {
        $wp_content_lang = Setup::$root.'/wp/wp-content/languages';
        $new_lang = Setup::$root.'/content/languages';

        if (file_exists($wp_content_lang)) {
            if (file_exists($new_lang)) {
                Setup::removeDir($new_lang);
            }

            rename($wp_content_lang, $new_lang);
            Setup::removeDir($new_lang.'/themes');
        }
    }

    /**
     * Remove a directory
     *
     * @param $dir
     */
    private static function removeDir($dir)
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);

            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (filetype($dir."/".$object) == "dir") {
                        Setup::removeDir($dir."/".$object);
                    } else {
                        unlink($dir."/".$object);
                    }
                }
            }

            reset($objects);
            rmdir($dir);
        }
    }

    /**
     * Remove the default wp-content directory.
     */
    private static function removeDefaultContentDir()
    {
        $removeDefaultContentDir = Setup::$composer->getConfig()->get('remove-default-content-dir');

        if ($removeDefaultContentDir) {
            Setup::removeDir(Setup::$root.'/wp/wp-content');
        }
    }

    private static function copyHtAccessFile() {

        $wpConfigSample_file = Setup::$root."/wp-config-sample.php";
        $wpConfig_file = Setup::$root."/wp-config.php";
        $htaccessSample_file = Setup::$root."/.htaccess-sample";
        $htaccess_file = Setup::$root."/.htaccess";

        if (!file_exists($wpConfig_file)) {
            copy($wpConfigSample_file, $wpConfig_file);
        }

        if (!file_exists($htaccess_file)) {
            copy($htaccessSample_file, $htaccess_file);
        }
    }

    /**
     * Attempt to create a config file.
     *
     * @return bool
     */
    private static function createConfigFile()
    {
        $env_file = Setup::$root."/.env";

        if (file_exists($env_file)) {
            return true;
        }

        if (!Setup::askConfirmation('<info>Create config file?</info> [<comment>Y,n</comment>]? ')) {
            return true;
        }

        $config_vars = array_map(function ($key) {
            $input = Setup::ask('What is the value of '.$key.'? ');

            return $key.'='.$input;
        }, Setup::$CONFIG_KEYS);

        $salts = array_map(function ($key) {
            return "$key='".Setup::generateSalt()."'";
        }, Setup::$AUTH_KEYS);

        if (touch($env_file)) {
            file_put_contents($env_file, implode("\n", $config_vars)."\n\n");
            file_put_contents($env_file, implode("\n", $salts), FILE_APPEND | LOCK_EX);
        } else {
            Setup::write("<error>An error occured while copying your .env file</error>");

            return false;
        }

        return true;
    }

    /**
     * Ask for confirmation to the user via console.
     *
     * @param string $question
     * @param bool $default
     * @return mixed
     */
    private static function askConfirmation($question, $default = false)
    {
        return Setup::$io->askConfirmation($question, $default);
    }

    /**
     * Ask the user a question.
     *
     * @param $question
     * @param $default
     *
     * @return mixed
     */
    private static function ask($question, $default = null)
    {
        return Setup::$io->ask($question, $default);
    }

    /**
     * Generate WordPress salts.
     *
     * @param int $length
     * @return string
     */
    private static function generateSalt($length = 64)
    {
        $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $chars .= '!@#$%^&*()';
        $chars .= '-_ []{}<>~`+=,.;:/?|';

        $salt = '';
        for ($i = 0; $i < $length; $i++) {
            $salt .= substr($chars, rand(0, strlen($chars) - 1), 1);
        }

        return $salt;
    }

    /**
     * Write a message to console.
     *
     * @param $messages
     * @param bool $newline
     * @param int $verbosity
     *
     * @return mixed
     */
    private static function write($messages, $newline = true, $verbosity = IOInterface::NORMAL)
    {
        return Setup::$io->write($messages, $newline, $verbosity);
    }
}